package provider

import scala.concurrent.{Future, Promise}
import scala.util.Try
import concurrent.ExecutionContext.Implicits.global

object ProviderUtils:
  /** Returns a new future that will be completed with the result of the first
    * future that completes.
    */
  def firstCompletedOf[T](fA: Future[T], fB: Future[T]): Future[T] =
    val p: Promise[T] = Promise()
    fA.andThen(p.tryComplete(_))
    fB.andThen(p.tryComplete(_))
    p.future

  /** Returns a new future that will be completed with the result of the first
    * future that completes successfuly. If the two futures fail, the returned
    * future fails with a `java.util.NoSuchElementException`.
    */
  def firstSuccessOf[T](fA: Future[T], fB: Future[T]): Future[T] =
    val p: Promise[T] = Promise()

    def isFailure(f: Future[T]) =
      f.isCompleted && f.value.forall(_.isFailure)

    def tryCompleteWithFailure: Unit =
      if isFailure(fA) && isFailure(fB) then
        p.tryFailure(new NoSuchElementException())

    def tryComplete(t: Try[T]) =
      if t.isSuccess then p.tryComplete(t)
      else tryCompleteWithFailure

    fA.andThen(tryComplete(_))
    fB.andThen(tryComplete(_))
    p.future
