package provider

import scala.concurrent.Future
import scala.concurrent.ExecutionContext.Implicits.global

import provider.ProviderUtils.firstSuccessOf

trait SearchQuery[T]:
  def matches(t: T): Boolean

case class MusicSearchQuery(
    artist: Option[String],
    album: Option[String],
    title: Option[String],
    year: Option[Int]
) extends SearchQuery[MusicEntry]:
  def matches(song: MusicEntry) =
    artist.forall(_ == song.artist) &&
      album.forall(_ == song.album) &&
      title.forall(_ == song.title) &&
      year.forall(_ == song.year)

class SearchProviderSync[Key, Entry](p: RacingProviderSync[Key, Entry])
    extends ProviderSync[SearchQuery[Entry], Set[Entry]]:
  def retrieveSync(query: SearchQuery[Entry]): Option[Set[Entry]] =
    val keys: Set[Key] =
      p.providerEU.retrieveKeysSync ++ p.providerUS.retrieveKeysSync
    Some((for
      k <- keys
      item <- p.providerEU.retrieveSync(k) ++ p.providerUS.retrieveSync(k)
      if query.matches(item)
    yield item).toSet)

  def retrieveKeysSync: Set[SearchQuery[Entry]] =
    throw UnsupportedOperationException(
      "Cannot retrieve keys of search provider"
    )

class SearchProvider[Key, Entry](p: RacingProvider[Key, Entry])
    extends Provider[SearchQuery[Entry], Set[Entry]]:
  def retrieve(query: SearchQuery[Entry]): Future[Set[Entry]] =
    ???

  def retrieveKeys: Future[Set[SearchQuery[Entry]]] =
    throw UnsupportedOperationException(
      "Cannot retrieve keys of search provider"
    )
