package provider

import TestDb.*

import java.util.concurrent.TimeoutException
import scala.concurrent.Await
import scala.concurrent.Future
import scala.concurrent.Promise
import scala.language.adhocExtensions

import concurrent.duration.DurationInt

// 15pts
class RacingProviderTest extends munit.FunSuite:
  val bp = RacingProvider(musicProviderEU, musicProviderUS)

  test("RacingProvider.retrieve should retrieve music from EU (2pts)"):
    val r = Await.result(bp.retrieve(MusicKey("21")), 1.second)
    assertEquals(r, dbContentEU(MusicKey("21")))

  test("RacingProvider.retrieve should retrieve music from US (2pts)"):
    val r = Await.result(bp.retrieve(MusicKey("17")), 1.second)
    assertEquals(r, dbContentUS(MusicKey("17")))

  test(
    "RacingProvider.retrieve should fail with NoSuchElementException if not available (2pts)"
  ):
    intercept[NoSuchElementException]:
      Await.result(bp.retrieve(MusicKey("42")), 1.second)

  test(
    "RacingProvider.retrieve should query both databases before waiting for results (3pts)"
  ):
    val mockEuP = Promise[Unit]()
    val mockEuDb = new Database(dbContentEU):
      override def retrieve(key: MusicKey): Future[MusicEntry] =
        mockEuP.trySuccess(())
        Future.never

    val mockUsP = Promise[Unit]()
    val mockUsDb = new Database(dbContentUS):
      override def retrieve(key: MusicKey): Future[MusicEntry] =
        mockUsP.trySuccess(())
        Future.never

    val bp = RacingProvider(mockEuDb, mockUsDb)
    bp.retrieve(MusicKey("42"))
    Await.result(mockEuP.future, 1.second)
    Await.result(mockUsP.future, 1.second)

  test(
    "RacingProvider.retrieve should return the entry from the first database that completes (2pts)"
  ):
    val mockEuPRes = Promise[MusicEntry]()
    val mockEuDb = new Database(dbContentEU):
      override def retrieve(key: MusicKey): Future[MusicEntry] =
        mockEuPRes.future

    val mockUsDb = new Database(dbContentUS):
      override def retrieve(key: MusicKey): Future[MusicEntry] =
        Future.never

    val bp = RacingProvider(mockEuDb, mockUsDb)
    val future = bp.retrieve(MusicKey("21"))
    mockEuPRes.success(dbContentEU(MusicKey("21")))
    val r = Await.result(future, 1.second)
    assertEquals(r, dbContentEU(MusicKey("21")))

  test("RacingProvider.retrieveKeys should retrieve keys from both databases (2pts)"):
    val r = Await.result(bp.retrieveKeys, 1.second)
    assertEquals(r, dbContentEU.keySet ++ dbContentUS.keySet)

  test(
    "RacingProvider.retrieveKeys should query both databases before waiting for results (2pts)"
  ):
    val mockEuP = Promise[Unit]()
    val mockEuDb = new Database(dbContentEU):
      override def retrieveKeys: Future[Set[MusicKey]] =
        mockEuP.trySuccess(())
        Future.never

    val mockUsP = Promise[Unit]()
    val mockUsDb = new Database(dbContentUS):
      override def retrieveKeys: Future[Set[MusicKey]] =
        mockUsP.trySuccess(())
        Future.never

    val bp = RacingProvider(mockEuDb, mockUsDb)
    bp.retrieveKeys
    Await.result(mockEuP.future, 1.second)
    Await.result(mockUsP.future, 1.second)
